//+------------------------------------------------------------------+
//|                                                        Ruler.mqh |
//|                                            Copyright 2010, Urain |
//|                            https://login.mql5.com/ru/users/Urain |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, Urain"
#property link      "https://login.mql5.com/ru/users/Urain"
//+------------------------------------------------------------------+
//| #include <Ruler.mqh>                                             |
//+------------------------------------------------------------------+
class CSearchArray
  {
private:
   int               QuickSearch(datetime element);// fast search
   int               SearchLess(datetime element); // search of an element
protected:
   //int               cnt;
   int               step;    // step of change of reserving as 2^step // set in the constructor
   int               reserved;// value of memory reserve
public:
   datetime          time[];  // array of data
   double            open[];  //--//
   double            high[];  //--//
   double            low[];   //--//
   double            close[]; //--//
   int               size;    // size of arrays
                     CSearchArray(void);
                    ~CSearchArray(void){};
   void              Free();                             // releasing the memory, zeroizing the reserve
   int               Resize(int size);                   // chaning the size of arrays
   int               SearchLessOrEqual(datetime element);// searching for an element with rounding to the time of bar
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CSearchArray::CSearchArray(void)
  {
//cnt=0;
   step=5;// set the step of change of reserving as 2^step
   reserved=0;
   size=0;
   Resize(0);
  }
//+------------------------------------------------------------------+
//| releasing the memory, zeroizing the reserve                      |
//+------------------------------------------------------------------+
void CSearchArray::Free()
  {
   size=0; reserved=0;
   ArrayFree(time);
   ArrayFree(open);
   ArrayFree(high);
   ArrayFree(low);
   ArrayFree(close);
  };
//+------------------------------------------------------------------+
//| chaning the size of arrays                                       |
//+------------------------------------------------------------------+
int CSearchArray::Resize(int size_)
  {
   size=size_;
   if((size>>step<<step)<reserved)
     {
      ArrayResize(time,size);
      ArrayResize(open,size);
      ArrayResize(high,size);
      ArrayResize(low,size);
      ArrayResize(close,size);
     }
   else
     {
      reserved=((size>>step)+1)<<step;
      int new_reserv=reserved-size;// amount of memory to be additionally reserved
      ArrayResize(time,size,new_reserv);
      ArrayResize(open,size,new_reserv);
      ArrayResize(high,size,new_reserv);
      ArrayResize(low,size,new_reserv);
      ArrayResize(close,size,new_reserv);
     }
   return(size);
  }
//+------------------------------------------------------------------+
//| searching for an element with rounding to the time of bar        |
//+------------------------------------------------------------------+
int CSearchArray::QuickSearch(datetime element)
  {
   int  i,j,m=-1;
   datetime temp;
//--- search
   i=0;
   j=size-1;
   while(j>=i)
     {
      //--- ">>1" is quick division by 2
      m=(j+i)>>1;
      if(m<0 || m>=size) break;
      temp=time[m];
      if(temp==element) break;
      if(temp>element)  j=m-1;
      else              i=m+1;
     }
//---
   return(m);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CSearchArray::SearchLess(datetime element)
  {
   int pos;
//--- checking   
   if(size==0) return(-1);
   if(time[0]==element)return(0);
//--- searching
   pos=QuickSearch(element);

   while(time[pos]>=element)
      if(pos--==0) return(-1);
//---
   return(pos);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CSearchArray::SearchLessOrEqual(datetime element)
  {
   int pos=SearchLess(element);

//--- searching
   if(pos!=-1)
     {
      if(pos!=size-1 && time[pos+1]==element) return(pos+1);
      else                                    return(pos);
     }
//---
   return(-1);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CRulerData : public CSearchArray
  {
public:
   string            symbol;// symbol
   datetime          beg;   // start
   datetime          end;   // end
                     CRulerData(void){};
                    ~CRulerData(void){};
   //--- constructor with parameters // receiving data
   int               CRulerDataInit(string symbol,datetime beg_,datetime end_);
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CRulerData::CRulerDataInit(string symbol_,datetime beg_,datetime end_)
  {
   beg=beg_; end=end_; symbol=symbol_; // save parameters in internal variables
   int count=Bars(symbol,PERIOD_M1,beg,end);// get the amount of data
//---if no information is downloaded
   int ff=0;
   while(!IsStopped() && count<=0)// download information
     {
      count=Bars(symbol,PERIOD_M1,beg,end);
      if(count>0)break;
      Sleep(10);
      // if the time of getting of information exceed the 1 minute, stop working
      ff++; if(ff>600){Print("_No inforamtion for the symbol "+symbol);ExpertRemove(); break;}
     }

   MqlRates temp[];// declare a temporary array of structures
   Resize(CopyRates(symbol,PERIOD_M1,end,count,temp));
   
   time[0]=temp[0].time;
   open[0]=temp[0].open;
   high[0]=temp[0].high;
   low[0]=temp[0].low;
   close[0]=temp[0].close;
   int j=1;
   int Sizes=size;
   for(int i=1;i<Sizes;i++)
     {
      long raz=(long)temp[i].time-
      (long)time[j-1];
      if( raz>60 && raz<3600 )
        {
         size++; Resize(size);
         time[j]=(datetime)(time[j-1]+60);
         open[j]=close[j-1];
         high[j]=close[j-1];
         low[j]=close[j-1];
         close[j]=close[j-1];         
         i--;if(i<1)i=1;         
        }
      else
        {
         time[j]=temp[i].time;
         open[j]=temp[i].open;
         high[j]=temp[i].high;
         low[j]=temp[i].low;
         close[j]=temp[i].close;
        }        
      j++;
     }
     
   ArrayFree(temp);
   return(size);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CRulerTF : public CSearchArray
  {
private:
   CRulerData       *r;
   ENUM_TIMEFRAMES   period;
   long              PS;
   datetime          beg;   // start
   datetime          end;   // end
   datetime TimeBars(int bar){datetime temp[];CopyTime(r.symbol,period,bar,1,temp);return(temp[0]);};
public:
   int               bar;
                     CRulerTF(void){bar=0;period=0;};
                    ~CRulerTF(void){};

   void              Init(CRulerData  *rb){r=rb;};// binding data
   //--- constructor with parameters // initializaiton of period, range
   void              CRulerTFInit(ENUM_TIMEFRAMES period_,datetime beg_,datetime end_);
   //--- shift from the zero bar
   int               Schift(){return(Bars(r.symbol,period,r.beg,TimeBars(0)));};
   int               Area(){return(size);};    // area of the range
   ENUM_TIMEFRAMES   Period(){return(period);};// period of the object
   //--- setting new period
   void              SetPeriod(ENUM_TIMEFRAMES period_){period=period_;};
   //--- changing the range
   void              Update(datetime beg_,datetime end_){CRulerTFInit(period,beg_,end_);};
   //--- changing the period
   void              Update(ENUM_TIMEFRAMES period_){if(period_!=period)CRulerTFInit(period_,beg,end);};
   int               BarsByTime(datetime time);// get bar by time
   //--- get bar by time and memorize the result
   void              BarsByTime_Set(datetime time){bar=BarsByTime(time);};
   datetime          TimeByBars(int bar);      // get time by bar
   datetime          CastToCurr(datetime time);// casting of time to a current period 
   datetime          NextTime(datetime time);  // time of the next bar
   datetime          PrevTime(datetime time);  // time of the previous bar

  };
//+------------------------------------------------------------------+
//| cut 1 data into the specified periods                           |
//+------------------------------------------------------------------+
void CRulerTF::CRulerTFInit(ENUM_TIMEFRAMES period_,datetime beg_,datetime end_)
  {
   beg=(beg_>=r.end || beg_<=r.beg)?beg_:r.beg;   // start
   end=(end_>=r.end || end_<=r.beg)?end_:r.end;   // end
//--- if the start and end point are the same
   if(beg_==r.beg && end_==r.end)
     {
      if(period_==period)return;// exit if the period hasn't changed
      if(period_==PERIOD_M1)// if the period is M1 rewrite the 1 structure
        {
         Resize(r.size);
         for(int i=0;i<size;i++)
           {
            time[i] =r.time[i];
            open[i] =r.open[i];
            high[i] =r.high[i];
            low[i]  =r.low[i];
            close[i]=r.close[i];
           }
         SetPeriod(period_);
         PS=PeriodSeconds(period);
         return;
        }
     }

   SetPeriod(period_);
   PS=PeriodSeconds(period);
   if(period==0){Print("Error of setting Period "+__FUNCTION__);return;}

   if(size!=0)Free();
   int i=r.SearchLessOrEqual(beg),j,tempH,tempL;
   int c=r.SearchLessOrEqual(end);
   int Per=MinuteTF(period);
   while(!IsStopped())// cut the 1 data into specified periods
     {
      if(i>=r.size || i>=c){break;}// exit if it is the end of history
      long times=r.time[i]/PS*PS;
      if(r.time[i]>=times)// if the M1 time contains the specified period
        {
         size=Resize(size+1);// increase the size of receiving arrays
         time[size-1]=(datetime)times;// assign time
         //--- find the next bar of the specified period
         j=1;
         if(i+Per<r.size)// if the search doesn't go out of range of the array
           {
            if(r.time[i+Per]==r.time[i]+PS){j=Per;}// check if we're lucky
            else// if the next bar is not within a period, search
              {
               for(j=1;j<=Per;j++)// search for the next date
                  if(r.time[i+j]/PS*PS==times+PS)break;// exit if it is found
              }
            //--- assign data
            open[size-1]=0; high[size-1]=0; low[size-1]=0; close[size-1]=0;// initialize memory
            open[size-1]=r.open[i];
            tempH=ArrayMaximum(r.high,i,j); high[size-1]=r.high[tempH];
            tempL=ArrayMinimum(r.high,i,j); low[size-1]=r.low[tempL];
            close[size-1]=r.close[i+j-1];
           }
         i+=j;// move to the next bar of the specified period
        }
      else i++;// move by 1 step
     }
  }
//+------------------------------------------------------------------+
//| get bar by time                                                  |
//+------------------------------------------------------------------+
int CRulerTF::BarsByTime(datetime time_)
  {
   datetime time=(datetime)(time_/PS*PS);
   if(time<beg)return(0);// check whether we're out of range
   if(time>end){ return(size-1);}// check whether we're out of range
   int res=SearchLessOrEqual(time);
   if(res==-1)BarsByTime(time-PS);
   return(res);
  }
//+------------------------------------------------------------------+
//| get time by bar                                                  |
//+------------------------------------------------------------------+
datetime CRulerTF::TimeByBars(int bar)
  {
   if(bar<0)return(time[0]);// check whether we're out of range
   if(bar>=size)return(time[size-1]);// check whether we're out of range
   return(time[bar]);
  }
//+------------------------------------------------------------------+
//| casting time to the current period                               |
//+------------------------------------------------------------------+  
datetime CRulerTF::CastToCurr(datetime time)
  {
   return(TimeByBars(BarsByTime(time)));
  }
//+------------------------------------------------------------------+
//| time of a next bar                                               |
//+------------------------------------------------------------------+
datetime CRulerTF::NextTime(datetime time)
  {
   return(TimeByBars(BarsByTime(time)+1));
  }
//+------------------------------------------------------------------+
//| time of previous bar                                             |
//+------------------------------------------------------------------+
datetime CRulerTF::PrevTime(datetime time)
  {
   return(TimeByBars(BarsByTime(time)-1));
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
